# log.pl -- tool for displaying stats from the server log
# Albert Dvornik (bert@athena.mit.edu)
#
# $Id: log.pl,v 1.1 93/07/06 23:00:21 bert Exp Locker: bert $
#

$map{'log-hosts'} = '&do_log("$rest?$query")';

sub do_log {
    local($args) = @_;
    local($hmin,$lmin,$mmin,$imin,$fmin,$dmin) = (1,1,1,1,10,1); # minimum values for an entry to be displayed

    local($dig) = "/var/local/www/logs/digest";
    local($d_stats,$d_hosts,$d_long,$d_net,$d_mitnet,$d_date,$d_week,$d_files,$d_status) =
	("$dig/stats","$dig/hosts","$dig/long","$dig/net","$dig/mitnet","$dig/date","$dig/week","$dig/files","$dig/status");

    local(*LOG);		# a hack to localize filehandles
    local($restart,$k,$name,$count,$clist,$query,$lno,$logtime,$digtime);

    local(%unmonth) = ("Jan",0,"Feb",1,"Mar",2,"Apr",3,"May",4,"Jun",5,"Jul",6,
		       "Aug",7,"Sep",8,"Oct",9,"Nov",10,"Dec",11);

    local(%unday) = ("Sun",0,"Mon",1,"Tue",2,"Wed",3,"Thu",4,"Fri",5,"Sat",6);

    alarm($time_out);

    {
	($fhostname ne "W20-frobozz.MIT.EDU") && (&dbg_info, next);

	($args =~ m,\?(.+),) || ($query="info", &srv_info, next);
	$query=$1;

	($query =~ s/\+(.*)$//) &&
	    ((($count=$1) <= 0) &&
	     &error('bad_request',"log?$query+$count: argument after '+' must be a positive number"));

	($query eq "hosts") &&  ($hmin=($count?$count:$hmin), &head, &hosts,  &foot, next);
	($query eq "long") &&   ($lmin=($count?$count:$lmin), &head, &long,   &foot, next);
	($query eq "files") &&  ($fmin=($count?$count:$lmin), &head, &files,  &foot, next);
	($query eq "net") &&    ($imin=($count?$count:$imin), &head, &net,    &foot, next);
	($query eq "mitnet") && ($mmin=($count?$count:$mmin), &head, &mitnet, &foot, next);
	($query eq "date") &&   ($dmin=($count?$count:$dmin), &head, &date,   &foot, next);

	$count && &error('bad_request',"log?$query+$count: unknown query type");

	($query eq "info") &&                                (&srv_info, next);
	($query eq "weird") &&                               (&head, &weird,  &foot, next);
	($query eq "status") &&                              (&head, &status, &foot, next);
	($query eq "week") &&                                (&head, &week,   &foot, next);

	($query =~ /^\//) &&                                 (&head, &file_query,   &foot, next);
	($query =~ /^18\.[0-9]+\.[0-9]+\.\*$/) &&            (&head, &mitnet_query, &foot, next);
	($query =~ /^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$/) &&    (&head, &ip_query,     &foot, next);
	($query =~ /^[0-9]+\.[0-9]+\.[0-9]+\.\*$/) &&        (&head, &net_query,    &foot, next);
	($query =~ /^[0-9]+\.[0-9]+\.\*\.\*$/) &&            (&head, &net_query,    &foot, next);
	($query =~ /^[0-9]+\.\*\.\*\.\*$/) &&                (&head, &net_query,    &foot, next);
	($query =~ /^[a-zA-Z0-9\.\-\_]+$/) &&                (&head, &host_query,   &foot, next);

	&error('bad_request',"log?$query: unknown query type");
    }

    alarm(0);


#
# counts are done in a cron job to prevent lossage and high loads
#

    sub get_stats {
	open (LOG, $d_stats) || &error('internal_error',"can't open digest from $d_stats");
	$logtime = <LOG>;
	$restart = <LOG>;
	$digtime = <LOG>;
	$lno = <LOG>;
	$dens = <LOG>;
	close (LOG);
    }

#
# global beginning, ending, and stat page
#

    sub head {
	&get_stats;
	print "<ISINDEX><TITLE>WWW.MIT.EDU server usage information ($query) </TITLE>\n";
	print "<H1>WWW.MIT.EDU Server Usage</H1>\n";
	print "<H3>logfile span: $logtime - $digtime</H3>\n";
    }

    sub foot {&srvstat;}

    sub srvstat {
        print "<H3>Statistics since $logtime</H3><P>\n";
	print "Total: $lno<P>\n";
	print "$dens<P>\n";
	print "Server last restarted: $restart<P>\n";
    }

#
# various queries on the counted data
#

    sub file_query {
	open (LOG, $d_files) || &error('internal_error',"can't open digest from $d_files");
	local($found)=0;
	while (<LOG>) {
            chop;
            ($k,$name) = unpack("a11 a*",$_);
	    if ($name eq $query) {
		print "<H2>file $query</H2>\n";
		print "file has been requested $k times.<P>\n";
		$found=1;
	    }
	}
	if (!$found) {
	    print "No matching files found.<P>\n"; 
	    &info;
	}
	close(LOG);
    }

    sub net_query {
	open (LOG, $d_net) || &error('internal_error',"can't open digest from $d_net");
	local($found)=0;
	while (<LOG>) {
            chop;
            ($k,$name) = unpack("a11 a*",$_);
	    if ($name eq $query) {
		print "<H2>net $query</H2>\n";
		print "hosts on this net have originated $k requests.<P>\n";
		$found=1;
	    }
	}
	if (!$found) {
	    print "No matching networks found.<P>\n"; 
	    &info;
	}
	close(LOG);
    }

    sub mitnet_query {
	open (LOG, $d_mitnet) || &error('internal_error',"can't open digest from $d_mitnet");
	local($found)=0;
	while (<LOG>) {
            chop;
            ($k,$name) = unpack("a11 a*",$_);
	    if ($name eq $query) {
		print "<H2>subnet $query</H2>\n";
		print "hosts on this subnet have originated $k requests.<P>\n";
		$found=1;
	    }
	}
	if (!$found) {
	    print "No matching subnets found.<P>\n"; 
	    &info;
	}
	close(LOG);
    }

    sub host_query {
	$query =~ tr/A-Z/a-z/;
	open (LOG, $d_hosts) || &error('internal_error',"can't open digest from $d_hosts");
	local($found)=0;
	while (<LOG>) {
            chop;
            ($k,$name) = unpack("a11 a*",$_);
	    if ($name eq $query) {
		print "<H2>host $query</H2>\n";
		print "site has originated $k requests.<P>\n";
		$found=1;
	    }
	}
	if (!$found) {
	    print "No matching hosts found.<P>\n"; 
	    &info;
	}
	close(LOG);
    }

    sub ip_query {
	open (LOG, $d_hosts) || &error('internal_error',"can't open digest from $d_hosts");
	local($found)=0;
	while (<LOG>) {
            chop;
            ($k,$name) = unpack("a11 a*",$_);
	    if ($name eq $query) {
		print "<H2>host $query</H2>\n";
		print "site has originated $k requests.<P>\n";
		$found=1;
	    }
	}
	if (!$found) {
	    print "No matching hosts found.<P>\n"; 
	    &info;
	}
	close(LOG);
    }

#
# other formatting filters
#

    sub hosts {
	print "<H2>Client Hosts (condensed)</H2><P>\n";
	print "<PRE>#requests  hostname\n-----------------------\n";
	$count=0; $clist=0;
	for $k (sort {$hstat{$b} <=> $hstat{$a}} keys %hstat) {
	    if ($hstat{$k} >= $hmin) {
		printf "%9d  %s\n",$hstat{$k},$k;
		$clist++;
	    }
	    $count++;
	}
	print "\ntotal: $clist entr".(($clist==1)?"y":"ies")." listed (out of $count)</PRE><P>\n";
	print "Click here for <A HREF=\"log?long+$hmin\">long</A> list.<P>\n" unless $show{'long'};
    }

    sub long {
	print "<H2>Client Hosts (long list)</H2><P>\n";
	print "<PRE>#requests  hostname\n-----------------------\n";
	$count=0; $clist=0;
	for $k (sort {$lhstat{$b} <=> $lhstat{$a}} keys %lhstat) {
	    if ($lhstat{$k} >= $lmin) {
		printf "%9d  %s\n",$lhstat{$k},$k;
		$clist++;
	    }
	    $count++;
	}
	print "\ntotal: $clist host".(($clist==1)?"":"s")." listed (out of $count)</PRE><P>\n";
    }

    sub net {
	print "<H2>Client IP Networks</H2><P>\n";
	print "<PRE>#requests  network\n-----------------------\n";
	$count=0; $clist=0;
	for $k (sort {$ipstat{$b} <=> $ipstat{$a}} keys %ipstat) {
	    if ($ipstat{$k} >= $imin) {
		printf "%9d  %s\n",$ipstat{$k},$k;
		$clist++;
	    }
	    $count++;
	}
	print "\ntotal: $clist net".(($clist==1)?"":"s")." listed (out of $count)</PRE><P>\n";
    }

    sub mitnet {
	print "<H2>Client Subnets on MITnet</H2><P>\n";
	print "<PRE>#requests  MIT subnet\n-----------------------\n";
	$count=0; $clist=0;
	for $k (sort {$mitstat{$b} <=> $mitstat{$a}} keys %mitstat) {
	    if ($mitstat{$k} >= $mmin) {
		printf "%9d  %s\n",$mitstat{$k},$k;
		$clist++;
	    }
	    $count++;
	}
	print "\ntotal: $clist subnet".(($clist==1)?"":"s")." listed (out of $count)</PRE><P>\n";
    }

    sub files {
	print "<H2>File transfers</H2><P>\n";
	print "<PRE>#requests  pathname\n-----------------------\n";
	$count=0;
	for $k (sort {$fstat{$b} <=> $fstat{$a}} keys %fstat) {
	    if ($fstat{$k} >= $fmin) {
		printf "%9d  %s\n",$fstat{$k},$k;
		$clist++;
	    }
	    $count++;
	}
	print "\ntotal: $clist file".(($clist==1)?"":"s")." listed (out of $count)</PRE><P>\n";
    }

    sub weird {
	print "<H2>Weird server commands</H2><P>\n";
	print "<PRE>";
	for $k (keys %weird_cmds) {
	    printf "%4d '%s'\n",$weird_cmds{$k},$k;
	    $clist++;
	}
	print "</PRE><P>\n";
    }

    sub status {
	print "<H2>Server status log</H2><P>\n";
	print "<PRE>".join("",@servlog)."</PRE><P>\n";
    }

    sub date {
	print "<H2>Top transfer days (GMT)</H2><P>\n";
	print "<PRE>#requests  date\n-----------------------\n";
	$count=0;
	for $k (sort {$date{$b} <=> $date{$a}} keys %date) {
	    if ($date{$k} >= $dmin) {
		printf "%9d  %s\n",$date{$k},$k;
		$clist++;
	    }
	    $count++;
	}
	print "\ntotal: $clist date".(($clist==1)?"":"s")." listed (out of $count)</PRE><P>\n";
    }

    sub week {
	&daytime;
	print "<H2>Transfers by weekday</H2><P>\n";
	print "<PRE>#requests  req/hr  weekday\n---------------------------------\n";
	printf "%9d  %6.2f  Monday     (total %5.1f hours)\n",$wday[1],$daytime[1]?$wday[1]/$daytime[1]:0,$daytime[1];
	printf "%9d  %6.2f  Tuesday    (total %5.1f hours)\n",$wday[2],$daytime[2]?$wday[2]/$daytime[2]:0,$daytime[2];
	printf "%9d  %6.2f  Wednesday  (total %5.1f hours)\n",$wday[3],$daytime[3]?$wday[3]/$daytime[3]:0,$daytime[3];
	printf "%9d  %6.2f  Thursday   (total %5.1f hours)\n",$wday[4],$daytime[4]?$wday[4]/$daytime[4]:0,$daytime[4];
	printf "%9d  %6.2f  Friday     (total %5.1f hours)\n",$wday[5],$daytime[5]?$wday[5]/$daytime[5]:0,$daytime[5];
	printf "%9d  %6.2f  Saturday   (total %5.1f hours)\n",$wday[6],$daytime[6]?$wday[6]/$daytime[6]:0,$daytime[6];
	printf "%9d  %6.2f  Sunday     (total %5.1f hours)\n",$wday[0],$daytime[0]?$wday[0]/$daytime[0]:0,$daytime[0];
	print "\n</PRE><P>\n";
    }

    sub info {
	print <<"EndOfInfo";
Click <A HREF="log?info">here</A> for more information
on this service and its use.<P>
EndOfInfo
    }

    sub dbg_info {
	print <<"EndOfDbg";
<ISINDEX><TITLE>WWW.MIT.EDU server usage information (info)</TITLE>
<H1>WWW.MIT.EDU Server Usage</H1>
<H2>Log Parser Is Not Available</H2>
The Plexus WWW server log parser is temporarily unavailable, because
bert is waiting for the new version of the server before he does 
his fixes and stuff since he doesn't want to do it twice.<P>
Sources will be available here about then too.<P>
Sorry for all this, but I have a real job these days... =P
EndOfDbg
    }

    sub srv_info {
	print <<"EndOfInfo";
<ISINDEX><TITLE>WWW.MIT.EDU server usage information (info) </TITLE>
<H1>WWW.MIT.EDU Server Usage</H1>
<H2>Log Parser Information</H2>
Welcome to the SIPB WWW server log.  You can look up various server stats from here.<P>
This node has been set up as a <STRONG>searchable index</STRONG>.  Useful keywords you
can search for are:<P>
<UL>
<LI><EM>hostnames</EM><P>Searching for a specific hostname (or IP address) will give
information on the requests this server has gotten from that host.  Hostname should refer
to the host's primary name.  If it isn't fully qualified, .MIT.EDU domain is assumed.<P>
Searching for the keyword
<CODE>"<A HREF="log?hosts">hosts</A>+<EM>n</EM>"</CODE> will give a
compact list of the sites that have contacted the server at least
<CODE><EM>n</EM></CODE> times.  (The default is currently set to
$hmin.)  All the *.MIT.EDU sites will be listed in a single entry.<P>
Searching for the keyword
<CODE>"<A HREF="log?long">long</A>+<EM>n</EM>"</CODE> will give a
complete list of all the hosts that have contacted the server at least
<CODE><EM>n</EM></CODE> times.  (The default is currently set to
$lmin.)<P>
<LI><EM>network IP numbers</EM><P>Searching for net IP numbers (in the
form <CODE>"num.*.*.*"</CODE> for type A, <CODE>"num.num.*.*"</CODE> for
type B or <CODE>"num.num.num.*"</CODE> for type C networks) will show the
total usage for all hosts on that net.<P>
Searching for the keyword <CODE>"<A
HREF="log?net">net</A>+<EM>n</EM>"</CODE> will give a complete list of
all the IP networks that have totalled at least
<CODE><EM>n</EM></CODE> requests to the server.  (The default is
currently set to $imin.)<P>
<LI><EM>MIT subnet IP numbers</EM><P>Searching for subnet IP numbers
(in the form <CODE>"18.num.num.*"</CODE>) will show the total usage for
all hosts on that subnet.<P>
Searching for the
keyword <CODE>"<A HREF="log?mitnet">mitnet</A>+<EM>n</EM>"</CODE> will
give a complete list of all the MIT subnets that have totalled at
least <CODE><EM>n</EM></CODE> requests to the server.  (The default is
currently set to $mmin.)<P>
<LI><EM>filenames</EM><P>Searching for a specific pathname will display total number of
times the file has been requested from this server.  The name should omit the initial
<CODE>http://www.mit.edu:8001</CODE> string and start with a '/'.  Queries of the form 
<EM>/path?query</EM> get logged under <EM>/path</EM>.<P>
Searching for the keyword
<CODE>"<A HREF="log?files">files</A>+<EM>n</EM>"</CODE> will give a
list of all the files that have been requested at least
<CODE><EM>n</EM></CODE> times.  (The default is currently set to
$fmin.)  Index queries of the form <EM>/path?query</EM> get logged
under <EM>/path</EM>.<P>
<LI><EM>Server status</EM><P>Searching for tke keyword
<CODE>"<A HREF="log?status">status</A>"</CODE> will show the list of the server
startups and restarts.<P>
Searching for tke keyword 
<CODE>"<A HREF="log?date">date</A>"</CODE> will show the requests
sorted by Greenwich Mean date.  (The server currently logs times in GMT, and it would
be a real kludge to fix it.)<P>
Searching for tke keyword
<CODE>"<A HREF="log?week">week</A>"</CODE> will show the data on
requests sorted by the (localtime) weekday.  (This data is interesting because it
reveals more of patterns in the mysterious secret life of
<A HREF="http://iicm.tu-graz.ac.at:80/Cjargon">hackers</A>.  Hey, maybe
I can figure out a way to use the log to explore their mating habits... =)<P>
</UL>
Please send any <A HREF="/comment">comments</A> or suggestions to
<CODE>webmaster@mit.edu</CODE><P>
<address><a href="/people/bert.html">bert Dvornik</a></address>
EndOfInfo

    }
}
