#!/usr/local/bin/perl
# plexus.pl -- HyperText Transfer Protocol Daemon main loop
#
# $Id: plexus.pl,v 1.3 93/06/09 23:54:42 mkgray Exp $
#
# Originally by Marc VanHeyningen, March 1993
# Modified by Tony Sanders <sanders@bsdi.com>, April 1993
#
# This code forms the core of a multi-threaded HTTP deamon, with the
# primary emphasis being on responsiveness; thus, things like exec()
# and parsing are kept to a minimum when responding to requests.
#

die "plexus.pl must be called from plexus\n" unless $#ARGV == 3;

$http_config = shift;		# config file
$port = shift;			# port id (FYI only)
$running_as_root = shift;	# who started us
$s = shift;			# socket file descriptor
require "$http_config";

$ENV{'TZ'} = 'GMT';		# be network friendly

# require 'ioctl.pl';
require 'ctime.pl';

$sockaddr = 'S n a4 x8';
do 'sys/wait.ph';
do 'sys/file.ph';
eval 'sub LOCK_SH {0x01;}'	unless defined &LOCK_SH;
eval 'sub LOCK_EX {0x02;}'	unless defined &LOCK_EX;
eval 'sub WNOHANG {1;}'		unless defined &WNOHANG;

# load the files in @inclist, set in $http_config
while ($_ = shift @inclist) { require $_; }

if ($running_as_root) {
    open(LOG, ">>$http_log") || die "$http_log: $!";
} elsif ($< == 97){
    open(LOG, ">>$http_log") || die "$http_log: $!";
}
else
{
    open(LOG, ">-") || die "/dev/tty: $!";
}
select(LOG); $| = 1;
print LOG "----Server #$$ on port $port started at ", &ctime(time);

$restart_daemon = 0;
$SIG{'INT'} = "cleanup";
$SIG{'USR1'} = "restart_daemon";
$SIG{'CHLD'} = "reaper"; 
require "../src/run-edsc.pl";
&run_edsc();
open(S, "+>&$s") || die "reopen of socket filedescriptor failed: $!";

CONNECTION:
until ($restart_daemon) {

    # XXX: need to figure out how to restart daemon first try.

    # trap "interruped system call"
    eval '{ $addr = accept(NS, S) || die "accept: $!"; }';
    if($@) {
	next CONNECTION 
	    if($@ =~ /^accept: Interrupted system call/);
	print LOG "----Server exiting: $@\n";
	die "$@";
    }

    # fork immediately to prevent delays
    if(($child = fork()) == 0) {
	$SIG{'ALRM'} = "timeout_error";
	select(NS); $| = 1;
	alarm($http_timeout);
	$_ = <NS>; s/[ \t\r\n]*$//;
	&log;

	alarm($http_timeout);
	&process_input;
	close(NS);
	exit;
    }
    close(NS);
}

alarm($http_timeout);
flock(LOG, &LOCK_EX) if $running_as_root;
print LOG "----Server #$$ restarting at ", &ctime(time);
close(LOG);
alarm(0);
exec "$http_server", $http_config, $port, $running_as_root, $s;
die "$http_server: $!";  

sub restart_daemon { $restart_daemon++; }
sub reaper { while(waitpid(-1,&WNOHANG) > 0) { ; } }
sub timeout_error { &error('internal_error',
    "Server timed out after $http_timeout seconds."); }
sub cleanup { close(S); shutdown(S, 2); print STDOUT "\n\n"; exit; }
sub log {
    # log connection
    local($af, $port, $inetaddr) = unpack($sockaddr, $addr);
    local(@inetaddr) = unpack('C4', $inetaddr);
    ($fhostname, @garbage) = gethostbyaddr($inetaddr, 2); # added mkgray
    local($ctime) = &ctime(time); chop $ctime;
    flock(LOG, &LOCK_EX) if $running_as_root;
    printf(LOG "%s (%-15s) %s %s\n", $fhostname, join(".", @inetaddr), $ctime, $_);
    close(LOG);
    $fhostname = ($fhostname) ? $fhostname:join(".", @inetaddr); #added mkgray

}
sub error {
    local($status, $msg) = @_;
    $status = 'internal_error' unless defined($code{$status});
    &MIME_header($status, 'text/html');
    print NS "<HEADER>\n<TITLE>Server Error</TITLE>\n</HEADER>\n";
    print NS "<BODY>\n<H1>Server Error: $code{$status}</H1>\n$msg\n</BODY>\n";
    close(NS); exit(-1);
}
sub do_restart {
    &MIME_header('ok', 'text/html');
    print NS "<HEADER>\n<TITLE>Restarting server</TITLE>\n</HEADER>\n";
    print NS "<BODY>\n<H1>Restarting server</H1>\n</BODY>\n";
    close(NS); kill 30, getppid; exit;
}
sub debug {
    local($msg) = @_;
    return unless $debug;
    open(FOO, ">>/tmp/plexus.debug");
    select((select(FOO), $| = 1)[0]);
    print FOO $msg, "\n";
    close(FOO);
}

1;
