#!/afs/athena/contrib/perl/perl

require 'chat2.pl';

$map{'techinfo'} = '&do_techinfo("$rest?$query")';

sub do_techinfo {
    local($qpath,$query) = split(/\?/,@_[0]);
    local($pserv,$dtype,$dnum,$ed) = $qpath ? split(m@/@,$qpath) : ();

    local(@iserver) = split(/:/,$pserv);
    local($eol,$eoms,$eolm) = ("\012",".\015","\012.\015\012");

    local($status,$handle);
    local($timeout) = 10;

    $SIG{'INT'}='report';
    $SIG{'HUP'}='report';
    $SIG{'QUIT'}='report';

    {
	$qpath || (&srvlist, next);
	$ed    &&  &error('bad_request','Invalidly formatted request');
	$dtype || (&open_server(@iserver), &send_flags, &list_menu(0), &close_server, next);
	$dnum  ||  &error('bad_request','Invalidly formatted request');

	($dtype eq 'menu') && (&open_server(@iserver), &send_flags, &list_menu($dnum), &close_server, next);
	($dtype ne 'doc') && &error('bad_request','Invalidly formatted request');

	&open_server(@iserver);
	&send_flags;
	&get_doc($dnum);
	&close_server;
    }

    sub print_ret {
	print "ret: @_[0], message: $status\n";
    }

    sub open_server {
	$handle = &chat'open_port(@_)
	    || do {&error('not_found','Failed to establish connection with Techinfo server');};
	($status = &chat'expect($timeout, $eolm, '$`')) =~ s/^([^:]*)://;
	$1;
    }

    sub close_server {
	&chat'print("q$eol");
	&chat'close($handle);	# no need to wait
    }

    sub send_flags {
	&chat'print("O:2$eol");
	($status = &chat'expect($timeout, $eolm, '$`')) =~ s/^([^:]*)://;
        $1;
    }

    sub get_doc {
	local($doc,$l,$x) = @_[0];
	local($lvl,$node,$flags,$date,$topic,$title,$source,$locker,$path);
        &chat'print("s:$doc$eol");
        ($lvl,$node,$flags,$date,$topic,$title,$source,$locker,$path)
            = split(/:/,&chat'expect($timeout, $eolm, '$`'));
        print "<TITLE>$title ($source)</TITLE><H1>$title</H1>\n";
        &chat'print("t:$doc$eol");
        (local($chr,$sent,$mod) = (&chat'expect($timeout, $eol, '$`')
            =~ /^(.*) Total Characters :(.*) sent:This document was last modified on (.*)$/))
            || &error('internal_error','Unable to parse first line');
        print "Last modified on $mod<P><PRE>\n";
        $inp = &chat'expect($timeout, $eol, '$`');
        $l = length($inp);
        $x=2;
        while ($inp ne $eoms) {
            $inp=~s/&/&amp/g; $inp=~s/</&lt/g; $inp=~s/>/&gt/g;
	    print "$inp<P>\n";
	    $inp = &chat'expect($timeout, $eol, '$`');
            $l += length($inp)+1;
            $x++;
        }
        print "</PRE>Stat: $x reads, $l count, $sent sent\n";
    }

    sub flag_type {
	local($f) = @_;
	local($type,@desc) = ("menu");

	if ($f & 16)   {$type = "doc"; push(@desc,"text");}
	if ($f & 32)   {               push(@desc,"sysgen");}
	if ($f & 64)   {$type = "gif"; push(@desc,"gif");}
	if ($f & 128)  {$type = "doc"; push(@desc,"nostat");}
	if ($f & 256)  {$type = "lnk"; push(@desc,"srv_link");}
	if ($f & 512) {$type = "menu"; push(@desc,"menu");}
	if ($f & 1024) {               push(@desc,"binary");}
	if ($f & 2048) {               push(@dlist,"noindex");}
	($type,join(",",@desc));
    }


    sub list_menu {
	local($menu) = @_[0];
	local($lvl,$node,$flags,$date,$topic,$title,$source,$locker,$path);
        &chat'print("w:2:$menu:1$eol");
        ($status = &chat'expect($timeout, $eol, '$`')) =~ s/^([^:]*)://;
        local($itemno,$i)=$1;
        ($lvl,$node,$flags,$date,$topic,$title,$source,$locker,$path)
            = split(/:/,&chat'expect($timeout, $eol, '$`'));
        print "<TITLE>$title ($source)</TITLE><H1>$title</H1>\n";
        print "<UL>\n";
        for ($i=1; $i<$itemno; $i++) {
	    ($lvl,$node,$flags,$date,$topic,$title,$source,$locker,$path)
		= split(/:/,&chat'expect($timeout, $eol, '$`'));
            local($type,$fdesc) = &flag_type($flags);
            print "<LI><A HREF=\"/techinfo/$pserv/$type/$node\">$title</A> ($fdesc)\n";
        }
        print "</UL>\n";
        (&chat'expect($timeout, $eol, '$`') eq $eoms)
          || &error('known','Entry count does not match list.');
    }

    sub list_servers {
	local($z,$port,$srvid,$name,$email,$title,$shost,$comment,$pips);
        &chat'print("m:$eol");
        ($status = &chat'expect($timeout, $eol, '$`')) =~ s/^([^:]*)://;
        local($servno,$i)=$1;
        print "<UL>\n";
        for ($i=0; $i<$servno; $i++) {
	    ($z,$port,$srvid,$name,$email,$title,$shost,$comment,$pips)
		= split(/:/,&chat'expect($timeout, $eol, '$`'));
            print "<LI><A HREF=\"/techinfo/$shost:$port\">$title</A>\n";
        }
        print "</UL>\n";
        (&chat'expect($timeout, $eol, '$`') eq $eoms)
          || &error('known','Server count does not match list.');
    }

    sub srvlist {
	print <<"endOfList";
<TITLE>Techinfo Gateway</TITLE><H1>WWW-Techinfo Gateway</H1>

Welcome to the WWW-Techinfo gateway.  This gateway is written in Perl
as an extension of the Plexus 2.1 server.  Click on the names from
the list to visit corresponding Techinfo sites.<P>
Note that the gateway is far from finished or debugged: it still prints
out cryptic and has severe implementation deficiencies.
Comments should be sent to <A HREF="/people/bert.html">bert@mit.edu</A>.
<UL>
<LI><A HREF="/techinfo/ti-server.baylor.edu:9000">Baylor University</A>
<LI><A HREF="/techinfo/ti-server.baylor.edu:9001">Baylor University - test</A>
<LI><A HREF="/techinfo/techinfo.brown.edu:9000">Brown University</A>
<LI><A HREF="/techinfo/techinfo.dfci.harvard.edu:9000">Dana-Farber Cancer Institute</A>
<LI><A HREF="/techinfo/dunx1.ocs.drexel.edu:9000">Drexel University</A>
<LI><A HREF="/techinfo/tiserve.mit.edu:9000">MIT</A>
<LI><A HREF="/techinfo/marathon.mit.edu:9000">MIT - test</A>
<LI><A HREF="/techinfo/msuinfo.msstate.edu:9000">Mississippi State</A>
<LI><A HREF="/techinfo/xantos.uio.no:9000">University of Oslo</A>
<LI><A HREF="/techinfo/penninfo-srv.upenn.edu:9000">University of Pennsylvania</A>
</UL>
endOfList
    }

    sub report {
	&error('known',"Wheee!  I just got a SIG@_[0]!");
    }
}
