#!/usr/local/bin/perl
#
# log-parse.pl -- tool for parsing and digesting stats from the server log
# Albert Dvornik (bert@athena.mit.edu)
#
# This program should be run as a cron job.
# The suggested frequency is between once an hour and every 15 minutes.
#
# $Id: log-parser.pl,v 1.6 93/07/09 00:56:19 bert Exp Locker: bert $
#

#
# local specifications
#

$logs = "/var/local/www/logs";

$http_log =    $ARGV[0] || "$logs/log";
$digest_path = $ARGV[1] || "$logs/digest/";
$init_path =   $ARGV[2] || "$logs/digest/";
$tmp_log =     $ARGV[3] || "$logs/.log";
$save_log =    $ARGV[4] || "$logs/www_log";

$barf = "$logs/parser";

$ENV{"TZ"}="EDT";		# our timezone
$gmtdiff = 4;			# the difference between GMT and our timezone

$ps_cmd = (-f '/vmunix' ? "ps auxww" : "ps -ef");

#
# helpful stuff
#

require 'ctime.pl';
require 'timelocal.pl';

%unmonth = ("Jan",0,"Feb",1,"Mar",2,"Apr",3,"May",4,"Jun",5,"Jul",6,
	    "Aug",7,"Sep",8,"Oct",9,"Nov",10,"Dec",11);
%unday = ("Sun",0,"Mon",1,"Tue",2,"Wed",3,"Thu",4,"Fri",5,"Sat",6);

umask(002);

$#wday = 7;

#
# calls to all the places
#

if (! -e $http_log) {
    &warn("Logfile not found, just restarting server\n");
    &yank_servers;
    &kick_servers;
    exit(1);
}
if (-e $tmp_log) {&warn("Old temporary logfile exists!\n");}

&r_hosts ($init_path,"hosts");
&r_long  ($init_path,"long");
&r_net   ($init_path,"net");
&r_mitnet($init_path,"mitnet");
&r_files ($init_path,"files");
&r_weird ($init_path,"weird");
&r_status($init_path,"status");
&r_date  ($init_path,"date");
&r_week  ($init_path,"week");
&r_stats ($init_path,"stats");

rename($http_log,$tmp_log) || &barf("error renaming $http_log to $tmp_log: $!\n");
&yank_servers;
&kick_servers;
&count($tmp_log,$save_log);

&hosts ($digest_path,"hosts");
&long  ($digest_path,"long");
&net   ($digest_path,"net");
&mitnet($digest_path,"mitnet");
&files ($digest_path,"files");
&weird ($digest_path,"weird");
&status($digest_path,"status");
&date  ($digest_path,"date");
&week  ($digest_path,"week");
&stats ($digest_path,"stats");

unlink($tmp_log) || &barf("Can't unlink $tmp_log. $!.");

###################################
#
# process-manipulation stuff
#

sub yank_servers {
    @servers = ();
    open(PS, "$ps_cmd |") || &barf("Can't run ps. $!.\n");
    <PS>;
    while(<PS>) {
	($user,$pid) = split(" ",$_);
        if (($user eq "www") && 
	    (/\.\.\/src\/plexus.pl \.\.\/src\/plexus.conf/))
               {push(@servers,$pid);}
    }
    close(PS);
}

sub kick_servers {
    if (@servers) {
	kill 'USR1',@servers;
    } else {
	&warn("No server found, started one.\n");
	system "su www &";	# restart the server
    }
}

#
# helper stuff
#

sub cctime {
    local($tmp) = &ctime(@_);
    chop($tmp);
    $tmp;
}

sub servtime {
    local($wdays,$mons,$mday,$hms,$zone,$year) = split(/\s+/,@_[0]);
    local($hour,$min,$sec) = split(/:/,$hms);
    ($zone eq "GMT") ? &timegm($sec,$min,$hour,$mday,$unmonth{$mons},$year-1900)
	          : &timelocal($sec,$min,$hour,$mday,$unmonth{$mons},$year-1900);
}

sub daytime {
    local($wdays,$mons,$mday,$hms,$zone,$year) = split(/\s+/,$logtime);
    local($hour,$min,$sec) = split(/:/,$hms);
    local($wd) = $unday{$wdays};
    if ($zone eq "GMT") {$hour -= 4;}
    if ($hour < 0) {$hour += 24; $wd = ($wd+6)%7;}
    local($dtime) = 86400 - (3600*$hour + 60*$min + $sec);
    local($ctime) = time - $itime;

    @daytime=(0,0,0,0,0,0,0);
    $daytime[$wd] = $dtime/3600;

    $dtime = $dtime;

    while ($dtime < $ctime) {
	$wd = ($wd+1)%7;
	$daytime[$wd] += 24;
	$dtime += 86400;
    }
    $daytime[$wd] -= ($dtime - $ctime)/3600;
}

#
# this does the counting
#

sub count {
    open(LOG, @_[0])  || &barf("Can't open @_[0]: $!\n");
    open(SAVE, ">> @_[1]") || &barf("Can't open @_[1] for log output: $!\n");
    $itime = &servtime($logtime) unless $set_stats;

    while (<LOG>) {
	print SAVE;
	$lno++;
	if (/^----/) {
	    if (/^----Server #[0-9]+ on port [0-9]+ started at (.+)$/) {
		$restart=$1;
		if ($set_stats) {
		    $set_stats=0;
		    $logtime=$1;
		    $itime = &servtime($logtime);
		}
	    }
	    push(@servlog,$_); next;
	} else {
	    / \(([^ ]+) *\) / || &barf("line $lno in @_[0] is misformatted:<p>'$_'\n");
	    $host = $`;
	    $ip =   $1;
	    ($wday,$mon,$date,$time,$zone,$year,$cmd,$first,@rest) = split(" ",$');

	    $host =~ tr/A-Z/a-z/;
	    ($host =~ /^[^\.]+\.MIT\.EDU$/i) ? $hstat{"*.MIT.EDU"}++ : $hstat{"$host ($ip)"}++;
	    $lhstat{"$host ($ip)"}++;

	    ($cmd eq "GET") ? $fstat{($first=~/^(.*)\?/)?$1:$first}++ : $weird_cmds{$cmd}++;

	    @tmp=split(/\./,$ip);
	    {
		($ipstat{"$tmp[0].*.*.*"}++, next)       if ($tmp[0] < 128); # class A networks
		($ipstat{"$tmp[0].$tmp[1].*.*"}++, next) if ($tmp[0] < 192); # class B networks
		$ipstat{"$tmp[0].$tmp[1].$tmp[2].*"}++;                      # class C (and lower) networks
	    }
	    $mitstat{"$tmp[0].$tmp[1].$tmp[2].*"}++ if ($tmp[0]==18);

	    (($zone eq "GMT")&&($time < $gmtdiff)) ? $wday[($unday{$wday}+6)%7]++ : $wday[$unday{$wday}]++;
	    $date{"$wday $mon $date $year"}++;
	}
    }
    close(SAVE);
    close(LOG);
}

#
# input and output the stuff
#

sub hosts {
    open (DOUT, "> @_[0]@_[1]") || &barf("Can't open @_[0]@_[1] for output: $!\n");
    for $k (sort {$hstat{$b} <=> $hstat{$a}} keys %hstat) {
	printf DOUT "%9d  %s\n",$hstat{$k},$k;
    }
    close(DOUT);
}

sub r_hosts {
    if (open (DIN, "@_[0]@_[1]")) {
	while (<DIN>) {
	    chop;
	    ($cnt,$name) = unpack("a11 a*",$_);
	    $hstat{$name} = $cnt;
	}
	close(DIN);
    }
}

sub long {
    open (DOUT, "> @_[0]@_[1]") || &barf("Can't open @_[0]@_[1] for output: $!\n");
    for $k (sort {$lhstat{$b} <=> $lhstat{$a}} keys %lhstat) {
	printf DOUT "%9d  %s\n",$lhstat{$k},$k;
    }
    close(DOUT);
}

sub r_long {
    if (open (DIN, "@_[0]@_[1]")) {
	while (<DIN>) {
	    chop;
	    ($cnt,$name) = unpack("a11 a*",$_);
	    $lhstat{$name} = $cnt;
	}
	close(DIN);
    }
}

sub net {
    open (DOUT, "> @_[0]@_[1]") || &barf("Can't open @_[0]@_[1] for output: $!\n");
    for $k (sort {$ipstat{$b} <=> $ipstat{$a}} keys %ipstat) {
	printf DOUT "%9d  %s\n",$ipstat{$k},$k;
    }
    close(DOUT);
}

sub r_net {
    if (open (DIN, "@_[0]@_[1]")) {
	while (<DIN>) {
	    chop;
	    ($cnt,$name) = unpack("a11 a*",$_);
	    $ipstat{$name} = $cnt;
	}
	close(DIN);
    }
}

sub mitnet {
    open (DOUT, "> @_[0]@_[1]") || &barf("Can't open @_[0]@_[1] for output: $!\n");
    for $k (sort {$mitstat{$b} <=> $mitstat{$a}} keys %mitstat) {
	printf DOUT "%9d  %s\n",$mitstat{$k},$k;
    }
    close(DOUT);
}

sub r_mitnet {
    if (open (DIN, "@_[0]@_[1]")) {
	while (<DIN>) {
	    chop;
	    ($cnt,$name) = unpack("a11 a*",$_);
	    $mitstat{$name} = $cnt;
	}
	close(DIN);
    }
}

sub files {
    open (DOUT, "> @_[0]@_[1]") || &barf("Can't open @_[0]@_[1] for output: $!\n");
    for $k (sort {$fstat{$b} <=> $fstat{$a}} keys %fstat) {
	printf DOUT "%9d  %s\n",$fstat{$k},$k;
    }
    close(DOUT);
}

sub r_files {
    if (open (DIN, "@_[0]@_[1]")) {
	while (<DIN>) {
	    chop;
	    ($cnt,$name) = unpack("a11 a*",$_);
	    $fstat{$name} = $cnt;
	}
	close(DIN);
    }
}

sub weird {
    open (DOUT, "> @_[0]@_[1]") || &barf("Can't open @_[0]@_[1] for output: $!\n");
    for $k (keys %weird_cmds) {
	printf DOUT "%4d '%s'\n",$weird_cmds{$k},$k;
    }
    close(DOUT);
}

sub r_weird {
    if (open (DIN, "@_[0]@_[1]")) {
	while (<DIN>) {
	    chop; chop;
	    ($cnt,$dummy,$name) = unpack("a4 a2 a*",$_);
	    $weird_cmds{$name} = $cnt;
	}
	close(DIN);
    }
}

sub status {
    open (DOUT, "> @_[0]@_[1]") || &barf("Can't open @_[0]@_[1] for output: $!\n");
    print DOUT join("",@servlog);
    close(DOUT);
}

sub r_status {
    if (open (DIN, "@_[0]@_[1]")) {
	while (<DIN>) {
	    push(@servlog,$_);
	}
	close(DIN);
    }
}

sub date {
    open (DOUT, "> @_[0]@_[1]") || &barf("Can't open @_[0]@_[1] for output: $!\n");
    for $k (sort {$date{$b} <=> $date{$a}} keys %date) {
	printf DOUT "%9d  %s\n",$date{$k},$k;
    }
    close(DOUT);
}

sub r_date {
    if (open (DIN, "@_[0]@_[1]")) {
	while (<DIN>) {
	    chop;
	    ($cnt,$name) = unpack("a11 a*",$_);
	    $date{$name} = $cnt;
	}
	close(DIN);
    }
}

sub week {
    open (DOUT, "> @_[0]@_[1]") || &barf("Can't open @_[0]@_[1] for output: $!\n");
    &daytime;
    printf DOUT "%9d  %6.2f  Monday     (total %5.1f hours)\n",$wday[1],$daytime[1]?$wday[1]/$daytime[1]:0,$daytime[1];
    printf DOUT "%9d  %6.2f  Tuesday    (total %5.1f hours)\n",$wday[2],$daytime[2]?$wday[2]/$daytime[2]:0,$daytime[2];
    printf DOUT "%9d  %6.2f  Wednesday  (total %5.1f hours)\n",$wday[3],$daytime[3]?$wday[3]/$daytime[3]:0,$daytime[3];
    printf DOUT "%9d  %6.2f  Thursday   (total %5.1f hours)\n",$wday[4],$daytime[4]?$wday[4]/$daytime[4]:0,$daytime[4];
    printf DOUT "%9d  %6.2f  Friday     (total %5.1f hours)\n",$wday[5],$daytime[5]?$wday[5]/$daytime[5]:0,$daytime[5];
    printf DOUT "%9d  %6.2f  Saturday   (total %5.1f hours)\n",$wday[6],$daytime[6]?$wday[6]/$daytime[6]:0,$daytime[6];
    printf DOUT "%9d  %6.2f  Sunday     (total %5.1f hours)\n",$wday[0],$daytime[0]?$wday[0]/$daytime[0]:0,$daytime[0];
    close(DOUT);
}

sub r_week {
    if (open (DIN, "@_[0]@_[1]")) {
	$wday[1] = unpack("a11",<DIN>);
	$wday[2] = unpack("a11",<DIN>);
	$wday[3] = unpack("a11",<DIN>);
	$wday[4] = unpack("a11",<DIN>);
	$wday[5] = unpack("a11",<DIN>);
	$wday[6] = unpack("a11",<DIN>);
	$wday[0] = unpack("a11",<DIN>);
	close(DIN);
    }
}

sub stats {
    open (DOUT, "> @_[0]@_[1]") || &barf("Can't open @_[0]@_[1] for output: $!\n");
    local($dens) = $lno/(time-$itime);
    print DOUT &ctime($itime),&ctime(&servtime($restart)),&ctime($);
    print DOUT "$lno requests\n";
    printf DOUT "Average %5.2f requests/hour (%4.2f per minute, %6.4f per second... =)\n",
	3600*$dens,60*$dens,$dens;
    close(DOUT);
}

sub r_stats {
    if (open (DIN, "@_[0]@_[1]")) {
	$logtime = <DIN>;
	$restart = <DIN>;
	<DIN>;
	($lno) = split(" ",<DIN>);
	close(DIN);
    } else {$set_stats = 1;}
}

#
# critical error
#

sub warn {
    open (BARF, ">> $barf") || die "$0: Shit. Can't open error log. $!.";
    print BARF &cctime(time),": @_";
    close(BARF);
}

sub barf {
    &warn(@_);
    die "$0: @_";
}
