# decode.pl -- image selection decoder
#
# $Id: decode.pl,v 2.0 1993/05/19 22:40:19 sanders Exp $
#
# Tony Sanders <sanders@bsdi.com>, April 1993
#
# This is the support code for decoding images.
#
# For large images you'll want to scale the mask by some factor depending
# on how accurate the results must be.  It would be better to have a ppm
# style mask with each "color" being a different object.  If you write
# this let me know.  Currently you need a mask for each object.
#
# &region -- loads and tests a pixel, front end to &pixel and &loadmask
#
# &loadmask -- reads the image file into memory
# &pixel -- test if a pixel is set, image must already be loaded by &loadmask
# &rnd -- internal routine for &loadmask for rounding up to nearest byte

sub rnd { local($value, $incr) = @_; ($value + ($incr-1)) & ~($incr-1); }

sub loadmask {
    local(*image) = @_;
    local($bits);
    $image{'scanlen'} = &rnd($image{'width'}, 8);	# whole bytes
    open(BITS, $image{'filename'}) || die "$image{'filename'}: $!";
    sysread(BITS, $bits, $image{'scanlen'} * $image{'height'} / 8);
    close(BITS);
    $image{'bits'} = $bits;
}

sub pixel {
    local(*a, $x, $y) = @_;
    local($offset) = int((($y * $image{'scanlen'}) + $x)/8);
    local($byte) = unpack("c", substr($image{'bits'}, $offset, 4)) & 0xff;
    return (($byte & (1<<($x%8))) != 0);
}

sub region {
    local($file, $width, $height, $x, $y) = @_;
    local(*image);
    $image{'filename'} = $file;
    $image{'width'} = $width;
    $image{'height'} = $height;
    &loadmask(*image);
    return (&pixel(*image, $x, $y));
}

1;
