# dir.pl -- directory browser (with filter)
#
# $Id: dir.pl,v 1.1 1993/06/09 22:54:40 mkgray Exp $
#
# Tony Sanders <sanders@bsdi.com>, April 1993
#

do 'sys/file.ph';
eval 'sub LOCK_SH {0x01;}'      unless defined &LOCK_SH;

sub retrieve {
    local($path, $filter) = @_;
    local($npath) = "$path/$http_index";

    -e $path || &error('not_found', "$path does not exist");
    do { &send_file($npath, &deduce_content($npath)); return; } if -f $npath;
    do { &index_dir($path, $filter); return; } if -d $path;
    &send_file($path, &deduce_content($path));
}

sub deduce_content {
    local($path) = @_;
    local(@ext, $ext, $encoding) = split(/\./, $path);

    while (($ext = pop(@ext)) && (defined $encoding{$ext})) {
	$encoding .= '; ' if defined $encoding;
	$encoding .= $encoding{$ext};
    }
    &add_header(*out_headers, "Content-encoding: $encoding")
        if defined $encoding;
    return $ext{$ext} if defined $ext{$ext};
    return (-B $path ? $content_binary : $content_plain);
}

sub send_file {
    local($path, $content) = @_;

    alarm($http_timeout);
    open(FD, $path) || &error('not_found', "$path: $!");
    flock(FD, &LOCK_SH);
    if ($version) {
	# generate HTTP/1.0 header
	if ($content) {
	    &add_header(*out_headers,
		sprintf("Last-Modified: %s", &fmt_date((stat($path))[9])));
	    &MIME_header('ok', $content);
	}
    } else {
	local($ext) = $path =~ m/\.(\w+)$/;
	alarm($http_timeout);
	print '<PLAINTEXT>'
	    unless(defined $ext{$ext} || defined $encoding{$ext});
    }
    while(sysread(FD,$_,8192)!=0) { syswrite(NS,$_,length); alarm($http_timeout); }
    close(FD);
}

do 'sys/stat.ph';
eval 'sub S_ISDIR {
    local($m) = @_;
    eval "(($m & 0170000) == 0040000)";
}' unless defined &S_ISDIR;

sub index_dir {
    local($dir, $filter) = @_;
    local($count, $_) = 1;

    $http_indexdirs || &error('bad_request', "$dir is a directory");
    defined($hidden{$dir}) && &error('bad_request', "$dir is a directory");
    $filter = '.*' unless $filter;

    alarm($http_timeout);

    &MIME_header('ok', 'text/html');
    if (-f "$dir/$dir_header") {
	&send_file("$dir/$dir_header")
    } else {
	print <<EOT;
<HEADER>
<ISINDEX><TITLE>Index of /$dir: filter=$filter</TITLE>
</HEADER>
<BODY>
<H1>Directory Index of /$dir</H1>
Use the Search Keyword to pass a <A HREF="$http_perlexp">
perl regular expression</A> to be used as a filter.
<EM>Note:</EM> it cannot end with a special file suffix
like gif, just add a ``\$'' on the end (e.g., ``gif\$'').
See Also <A HREF="$http_dirdoc">The Directory Browser</A>.
EOT
    }

    print "<PRE>\n";
    $_ = $dir; while (chop ne '/' && length) { ; }
    printf "%39s%s", "", "<A NAME=$count HREF=\"/$_\">Parent Directory</A>\n"; $count++;

    # Get directory listing
    alarm($http_timeout);
    opendir(DH, "$dir") || &error('not_found', "$dir: $!");
    @dirs = sort(readdir(DH));
    while ($_ = shift @dirs) {
	alarm($http_timeout);
	next if /^\.$/ || /^\.\.$/;
	next if defined($hidden{"$dir/$_"});	# hidden from listing
	next unless /$filter/;
	local($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size,
	    $atime, $mtime, $ctime, $blksize, $blocks) = stat("$dir/$_");
	local($date) = &ctime($mtime); chop $date;
	printf "%s %9d ", $date, $size;
	print "<A NAME=$count HREF=\"/$dir/$_\">$_</A>"; $count++;
	&S_ISDIR($mode) && print "/";
	print "\n";
    }
    closedir(DH);

    print "</PRE>\n";

    if (-f "$dir/$dir_footer") {
        &send_file("$dir/$dir_footer");
    } else {
	print "</BODY>\n";
    }
}

1;
